﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace SetCBLUserImage
{
    public class CBL2OSUpdate
    {
        private List<IntelHexFile> _hexFiles;

        public IntelHexFile BaseCode
        {
            get
            {
                return _hexFiles.Where(t => t.DataType == 0x23).FirstOrDefault();
            }
        }

        public UserImage UserImage { get; private set; }

        private CBL2OSUpdate()
        {
            _hexFiles = new List<IntelHexFile>();
        }

        public static CBL2OSUpdate Load(string fileName)
        {
            return Load(fileName, false);
        }

        public static CBL2OSUpdate Load(string fileName, bool ignoreChecksums)
        {
            var ret = new CBL2OSUpdate();

            //Open the input file
            var input = new FileStream(fileName, FileMode.Open);

            while (input.Position < input.Length)
            {
                var file = IntelHexFile.Load(input, false, ignoreChecksums);
                switch (file.DataType)
                {
                    case 0x29:
                        //We're dealing with this one separately
                        ret.UserImage = new UserImage(file, ignoreChecksums);
                        break;
                    default:
                        //Not explicitly handled, so just add it
                        ret._hexFiles.Add(file);
                        break;
                }
            }

            //Close the input file
            input.Close();

            return ret;
        }

        public void Save(string fileName)
        {
            Save(fileName, false);
        }

        public void Save(string fileName, bool excludeOS)
        {
            var output = new FileStream(fileName, FileMode.Create);

            //Save the other stuff that was in the file
            foreach (var file in _hexFiles)
            {
                if (!excludeOS || file.DataType != 0x23)
                {
                    var data = file.GetSaveData();
                    output.Write(data, 0, data.Length);
                }
            }

            //Now save the user image
            var img = UserImage.GetSaveData();
            output.Write(img, 0, img.Length);

            output.Close();
        }
    }
}
